@@ -49,8 +49,11 @@ def check_key_in_dict(dictionary, keys):
 
 def preprocess_paths(paths: Union[List, str]):
     if isinstance(paths, list):
-        return [os.path.abspath(os.path.expanduser(path)) for path in paths]
-    return os.path.abspath(os.path.expanduser(paths)) if paths else None
+        return [path if path.startswith('gs://') else os.path.abspath(os.path.expanduser(path)) for path in paths]
+    elif isinstance(paths, str):
+        return paths if paths.startswith('gs://') else os.path.abspath(os.path.expanduser(paths))
+    else:
+        return None
 
 
 def nan_to_zero(input_tensor):
@@ -100,10 +103,10 @@ def read_bytes(path: str) -> tf.Tensor:
     return tf.convert_to_tensor(content, dtype=tf.string)
 
 
-def shape_list(x):
+def shape_list(x, out_type=tf.int32):
     """Deal with dynamic shape in tensorflow cleanly."""
     static = x.shape.as_list()
-    dynamic = tf.shape(x)
+    dynamic = tf.shape(x, out_type=out_type)
     return [dynamic[i] if s is None else s for i, s in enumerate(static)]
 
 
@@ -153,3 +156,14 @@ def log10(x):
 
 def get_reduced_length(length, reduction_factor):
     return tf.cast(tf.math.ceil(tf.divide(length, tf.cast(reduction_factor, dtype=length.dtype))), dtype=tf.int32)
+
+
+def count_non_blank(tensor: tf.Tensor, blank: int or tf.Tensor = 0, axis=None):
+    return tf.reduce_sum(tf.where(tf.not_equal(tensor, blank), x=tf.ones_like(tensor), y=tf.zeros_like(tensor)), axis=axis)
+
+
+def has_gpu_or_tpu():
+    gpus = tf.config.list_logical_devices("GPU")
+    tpus = tf.config.list_logical_devices("TPU")
+    if len(gpus) == 0 and len(tpus) == 0: return False
+    return True
